//------------------------------------------------------------------------ 2011-12-14 ----------
// cb[^[vO 
//
//  Clock 48MHz (20MHz crystal)
//-------------------------------------------------------- Written by Hiroyuki Iizuka ----------
#include <p18f4550.h>

#include <delays.h>
#include <stdio.h>
#include <stdlib.h>
#include <EEP.h>

#define VERSION  "Ver1.0"

//============@u[g[_[gȂ͂Rgɂ ==========================
// #define USE_BOOTLOADER
//========================================================================

//------------------- RtBO[V[h̐ݒ ----------------------------------------
// u[g[_[gpƂ̓u[g[_[̃RtBO[V[hgp̂ŁA̐ݒ͖
#pragma config USBDIV  = 2
#pragma config PLLDIV  = 5
#pragma config CPUDIV  = OSC1_PLL2
#pragma config FOSC    = HSPLL_HS
#pragma config PWRT    = ON         // Power-up Timer
#pragma config BOR     = ON
#pragma config BORV    = 3          // Brown-out Reset Voltage Minimum setting
#pragma config WDT     = OFF        // WDT disabled (control is placed on the SWDTEN bit)
#pragma config MCLRE   = OFF        // RE3 input pin enabled; MCLR pin disabled
#pragma config LVP     = OFF
#pragma config PBADEN  = OFF

#pragma config FCMEN   = OFF        // Fail-Safe Clock Monitor disabled
#pragma config VREGEN  = OFF        // USB voltage regulator disabled

#pragma config CP0     = OFF        // Block 0 (000800-001FFFh) is not code-protected
#pragma config CP1     = OFF        // Block 1 (002000-003FFFh) is not code-protected
#pragma config CP2     = OFF        // Block 2 (004000-005FFFh) is not code-protected
#pragma config CP3     = OFF        // Block 3 (006000-007FFFh) is not code-protected
#pragma config CPB     = OFF        // Boot block (000000-0007FFh) is not code-protected
#pragma config CPD     = OFF        // Data EEPROM is not code-protected
#pragma config WRT0    = OFF        // Block 0 (000800-001FFFh) is not write-protected
#pragma config WRT1    = OFF        // Block 1 (002000-003FFFh) is not write-protected
#pragma config WRT2    = OFF        // Block 2 (004000-005FFFh) is not write-protected
#pragma config WRT3    = OFF        // Block 3 (006000-007FFFh) is not write-protected
#pragma config WRTC    = OFF        // Configuration registers (300000-3000FFh) are not write-protected
#pragma config WRTB    = OFF        // Boot block (000000-0007FFh) is not write-protected
#pragma config WRTD    = OFF        // Data EEPROM is not write-protected
#pragma config EBTR0   = OFF        // Block 0 (000800-001FFFh) is not protected from table reads executed in other blocks
#pragma config EBTR1   = OFF        // Block 1 (002000-003FFFh) is not protected from table reads executed in other blocks 
#pragma config EBTR2   = OFF        // Block 2 (004000-005FFFh) is not protected from table reads executed in other blocks
#pragma config EBTR3   = OFF        // Block 3 (006000-007FFFh) is not protected from table reads executed in other blocks
#pragma config EBTRB   = OFF        // Boot block (000000-0007FFh) is not protected from table reads executed in other blocks


//----- ddoqnll ----------------------------------------------------
#pragma romdata eedata_scn=0xf00000 
rom char eedata_values[14] = {0xB8, 0x0B,   // 3000rpm
                                50,         // ddTu
                                50,         // [^[idTu
                              0xFF, 0x03,   // őf[eB[100
                                 0,         // 0:Free / 1:Brake / 2:Duty
                                 1,         // 0:CW / 1:CCW
                              0x30, 0x75,   // ROOOO  ő
                              0x64, 0x00,   // POO@@@@ŏ
                              0x64, 0x00    // POO@@@@Xebv
                             }; 
#pragma romdata

//----- ddoqnlAhX -----------------------------------------------------
#define eeaddr_initial_rpm_0       0   // ]   
#define eeaddr_initial_rpm_1       1   //
#define eeaddr_supply_voltage      2   // dd 5.0V ` 12.0V@dPO{li[
#define eeaddr_motor_voltage       3   // [^[id 1.0V `@dd@dPO{li[
#define eeaddr_max_duty_0          4   // max_duty = 1024 * MotorVoltage / SpplyVoltage
#define eeaddr_max_duty_1          5   //
#define eeaddr_control_mode        6   // Free / Brake / Duty
#define eeaddr_motor_direction     7   // CW / CCW
#define eeaddr_max_rpm_0           8   // target rpm upper limit
#define eeaddr_max_rpm_1           9   // 
#define eeaddr_min_rpm_0          10   // target rpm lower limit
#define eeaddr_min_rpm_1          11   // 
#define eeaddr_step_rpm_0         12   // GR[_[񂵂Ƃ̉]
#define eeaddr_step_rpm_1         13   // 

//----- H̍ői --------------------------------------------------------
#define MAX_SUPPLY_VOLTAGE       120   // = 12.0V
#define MIN_SUPPLY_VOLTAGE        50   // =  5.0V
#define MIN_MOTOR_VOLTAGE         10   // =  1.0V
#define MIN_RPM                  100   // setup lower limit
#define MAX_RPM                30000   // setup upper limit
#define MAX_STALL_TIME          5000   // [msec] ȏ㒷[^[~Ăً}~

//----- ʓIȒ` --------------------------------------------------------------
#define PORT_OUTPUT   0
#define PORT_INPUT    1

typedef unsigned char boolean; 
#define true  1
#define false 0

#define stop(void)    while(1) { ; }

//---------------- h^n|[g` -------------------------------------------------------------
//----- t -------
#define LCD_RS         LATAbits.LATA4
#define LCD_E          LATAbits.LATA5
#define LCD_DB4        LATAbits.LATA0
#define LCD_DB5        LATAbits.LATA1
#define LCD_DB6        LATAbits.LATA2
#define LCD_DB7        LATAbits.LATA3

#define LCD_RS_TRIS    TRISAbits.RA4
#define LCD_E_TRIS     TRISAbits.RA5
#define LCD_DB4_TRIS   TRISAbits.RA0
#define LCD_DB5_TRIS   TRISAbits.RA1
#define LCD_DB6_TRIS   TRISAbits.RA2
#define LCD_DB7_TRIS   TRISAbits.RA3

enum type_LCD_cursor { CURSOR_ON , CURSOR_OFF , BLINK_ON , BLINK_OFF };

void lcd_clear(void);
void lcd_enable_delay_us(unsigned short);
void lcd_init(void);
void lcd_cursor( enum type_LCD_cursor,  enum type_LCD_cursor );

//------------ tHgC^v^ --------------------------------------------------------
#define SENSOR_A   PORTBbits.INT0   
#define SENSOR_B   PORTBbits.INT1   

#define SENSOR_A_TRIS   TRISBbits.RB0   
#define SENSOR_B_TRIS   TRISBbits.RB1

#define sensor_A_changed            INTCONbits.INT0IF    // INT0 ͏Ɂ@high priority
#define sensor_A_int_enable         INTCONbits.INT0IE
#define sensor_A_int_rising_edge    INTCON2bits.INTEDG0

#define sensor_B_changed            INTCON3bits.INT1IF
#define sensor_B_int_enable         INTCON3bits.INT1IE
#define sensor_B_int_priority_high  INTCON3bits.INT1IP
#define sensor_B_int_rising_edge    INTCON2bits.INTEDG2

//---- Rkdc ---------------------------------------------------------------
#define LED1      LATDbits.LATD2    // 
#define LED2      LATDbits.LATD1    // 
#define LED3      LATDbits.LATD0    // 

#define LED1_TRIS TRISDbits.RD2
#define LED2_TRIS TRISDbits.RD1
#define LED3_TRIS TRISDbits.RD0

#define LED_ON  1
#define LED_OFF 0

//-----@[^[GR[_@ ------------------------------------------------
#define RENCODER_A   PORTEbits.RE0
#define RENCODER_B   PORTEbits.RE1

#define RENCODER_A_TRIS  TRISEbits.RE0
#define RENCODER_B_TRIS  TRISEbits.RE1

enum type_encoder_direction { CW, CCW, ENCODER_STOP };
enum type_encoder_direction rotary_encoder(void);
void rotary_encoder_init();

//-----@uU[ --------------------------------------------------------
#define BUZZER        LATBbits.LATB5
#define BUZZER_TRIS   TRISBbits.RB5

#define buzzer_on()   {BUZZER = 1;}
#define buzzer_off()  {BUZZER = 0;}

#define beep_pi()     buzzer_on();delay_ms(20);buzzer_off()

//----- gubW ------------------------------------------------------
#define MOTOR_P1A          LATCbits.LATC2
#define MOTOR_P1B          LATDbits.LATD5
#define MOTOR_P1C          LATDbits.LATD6
#define MOTOR_P1D          LATDbits.LATD7
#define MOTOR_SELECT       LATDbits.LATD4  // 1:select-B -> Free,Brake mode
                                           // 0:select-A -> Duty mode
#define TTL74HC157_G       LATDbits.LATD3 

#define MOTOR_P1A_TRIS     TRISCbits.RC2
#define MOTOR_P1B_TRIS     TRISDbits.RD5
#define MOTOR_P1C_TRIS     TRISDbits.RD6
#define MOTOR_P1D_TRIS     TRISDbits.RD7
#define MOTOR_SELECT_TRIS  TRISDbits.RD4 
#define TTL74HC157_G_TRIS  TRISDbits.RD3 

#define TTL74HC157_ENABLE  0
#define TTL74HC157_DISABLE 1

//----- XCb` --------------------------------------------------------
#define PUSH_SW1           PORTCbits.RC0
#define PUSH_SW2           PORTCbits.RC1
#define SLIDE_SW           PORTEbits.RE2

#define PUSH_SW1_TRIS      TRISCbits.RC0
#define PUSH_SW2_TRIS      TRISCbits.RC1
#define SLIDE_SW_TRIS      TRISEbits.RE2

#define SW_ON   0
#define SW_OFF  1

#define SW_ENABLE  1
#define SW_DISABLE 0

unsigned char Sw1_status;  // SW_ENABLE,SW_DISABLE
unsigned char Sw2_status;  // SW_ENABLE,SW_DISABLE
unsigned char Sw_timer;    // `^Oh~^C}[
  
//-------- ^C}[ ---------------------------------------------------------------
#define timer_start  1
#define timer_stop   0

//-------- TMR0i]vpj --------------------------------------------------
#define Timer0_overflowd              INTCONbits.TMR0IF
#define Timer0_interrupt_enable       INTCONbits.TMR0IE
#define Timer0_priority               INTCON2bits.TMR0IP
#define Timer0_enable                 T0CONbits.TMR0ON
#define Timer0_mode_8bit              T0CONbits.T08BIT
#define Timer0_use_external_clock     T0CONbits.T0CS
#define Timer0_prescale_NOT_assign    T0CONbits.PSA
#define Timer0_prescale_bit2          T0CONbits.T0PS2
#define Timer0_prescale_bit1          T0CONbits.T0PS1
#define Timer0_prescale_bit0          T0CONbits.T0PS0

//-------- TMR1 ------------------------ (gp) -----------------------------
#define Timer1_overflowd              PIR1bits.TMR1IF 
#define Timer1_interrupt_enable       PIE1bits.TMR1IE 
#define Timer1_prescale_bit0          T1CONbits.T1CKPS1
#define Timer1_prescale_bit1          T1CONbits.T1CKPS0
#define Timer1_use_external_clock     T1CONbits.TMR1CS
#define Timer1_enable                 T1CONbits.TMR1ON
#define Timer1_priority               IPR1bits.TMR1IP

//-------- TMR2iovlf[eB[pj -----------------------------------------------
#define Timer2_overflowd              PIR1bits.TMR2IF 
#define Timer2_interrupt_enable       PIE1bits.TMR2IE
#define Timer2_priority               IPR1bits.TMR2IP
#define Timer2_postscale_bit3         T2CONbits.T2OUTPS3
#define Timer2_postscale_bit2         T2CONbits.T2OUTPS2
#define Timer2_postscale_bit1         T2CONbits.T2OUTPS1
#define Timer2_postscale_bit0         T2CONbits.T2OUTPS0
#define Timer2_enable                 T2CONbits.TMR2ON
#define Timer2_prescale_bit1          T2CONbits.T2CKPS1
#define Timer2_prescale_bit0          T2CONbits.T2CKPS0
#define Timer2_prescale               T2CONbits.T2CKPS

//-------- TMR3iPԊԊu) ----------------------------------------------
#define Timer3_overflowd              PIR2bits.TMR3IF
#define Timer3_interrupt_enable       PIE2bits.TMR3IE
#define Timer3_priority               IPR2bits.TMR3IP
#define Timer3_use_external_clock     T3CONbits.TMR3CS
#define Timer3_enable                 T3CONbits.TMR3ON
#define Timer3_mode_16bit             T3CONbits.RD16
#define Timer3_not_sync_ext_clock     T3CONbits.T3SYNC
#define Timer3_prescale_bit1          T3CONbits.T3CKPS1
#define Timer3_prescale_bit0          T3CONbits.T3CKPS0

//-------- PORTB -------------------------------------------------------------
#define PortB_pullups_disable         INTCON2bits.NOT_RBPU

//----- 荞 -----------------------------------------------------------------------
#define High_priority 1
#define Low_priority  0

void InterruptVectorLow (void);
void InterruptVectorHigh (void);
void Low_priority_interrupt (void);
void High_priority_interrupt (void);

//----- Zp֐ -------------------------------------------------------------------
unsigned short pow_short( unsigned short, unsigned char );
unsigned short warizan_16x16(unsigned short, unsigned short );

//-----@^ϊp̋p ------------------------------------------------------------
typedef union {
  unsigned long  Lval;
  unsigned short Wval[2];
  unsigned char  Bval[4];
} long_union; 

typedef union {
  unsigned short Wval;
  unsigned char  Bval[2];
} short_union; 

//---- Ƀ[^[p̕ϐȂ ---------------------------------------
#define MIN_RPM          100
#define MAX_DUTY         1023
#define TMR_COUNT_60SEC  720000000      // @ 48MHz

enum type_control_mode      { MMODE_FREE, MMODE_BRAKE,  MMODE_DUTY };
enum type_motor_direction   { CW, CCW };
enum type_run_mode          { RUN, KEISOKU };
enum type_start_stop        { START, STOP };

struct {
 enum type_control_mode    control;
 enum type_motor_direction direction;
 enum type_run_mode        run_mode;
 enum type_start_stop      start;
} Mmode;

unsigned short TMR0_over_flow_count;
// unsigned short TMR1_over_flow_count;

#define RPM_DIV 60

unsigned short Pwm_duty;       // 0 - 1023 pwm duty
unsigned short Rpm_per_duty;   // rpm / duty
unsigned short Max_duty_limit; //   - 1023 pwm duty
unsigned short Target_rpm;
unsigned short Target_rpm_minus;
unsigned short Target_rpm_plus;
unsigned short Motor_stall_th; // [ms]@[^[~܂ĂƔf臒l 

//---- ɉ]\̂߂̕ϐȂ -------------------------------------------
#define DISPLAY_INTERVAL              300  // msec
#define DISPLAY_CHANGE_RPM_INTERVAL  2000  // msec
#define DISPLAY_TIMEOUT              1000  // msec  
#define DISPLAY_DUTY_INTERVAL         300  // msec  
unsigned short Display_interval_counter;   // up counter
unsigned short Display_change_rpm_counter; // down counter
unsigned short Display_timeout_counter;    // down counter
unsigned short Display_duty_counter;       // down counter
unsigned short Motor_stall_counter;        // up counter

enum { DISPLAY_WAIT, GET_TIMER, TIMER_COUNTING, DISPLAY_RPM } kaitenkei_stat;

long_union Raw_tmr_kaitenkei;  // kaitenkei

//----------------------------------------------------------------------------
// 荞݃xN^[ݒ
//----------------------------------------------------------------------------
#ifdef USE_BOOTLOADER
#pragma code InterruptVectorLow = 0x2018
#else
#pragma code InterruptVectorLow = 0x0018
#endif
void InterruptVectorLow (void)
{
  _asm
    goto Low_priority_interrupt   //jump to interrupt routine
  _endasm
}

#ifdef USE_BOOTLOADER
#pragma code InterruptVectorHigh = 0x2008
#else
#pragma code InterruptVectorHigh = 0x0008
#endif
void InterruptVectorHigh (void)
{
  _asm
    goto High_priority_interrupt  //jump to interrupt routine
  _endasm
}

#pragma code
#pragma interruptlow Low_priority_interrupt
#pragma interrupt    High_priority_interrupt

//-------------------------------------------------------------------------------
// delay  usec           Clock = 48Mhz
//-------------------------------------------------------------------------------
void delay_us(unsigned short delay)
{
  unsigned short i;

  for( i = 0 ; i <= delay ; i++ ) {
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
  }
}

//-------------------------------------------------------------------------------
// delay  msec           Clock = 48Mhz
//-------------------------------------------------------------------------------
void delay_ms(unsigned short delay)
{
  unsigned short i;

  for( i = 0 ; i <= delay ; i++ ) {
    Delay1KTCYx(12);
  }
}

//-------------------------------------------------------------------------------
// tfBXvCɂSrbgM@@v`hs
//-------------------------------------------------------------------------------
void lcd_send_nibble(unsigned char nibble, unsigned short nsec )
{
  unsigned char i;
  unsigned char bit_val;

  if ( (nibble & 0x01) == 0x01 ) { LCD_DB4 = 1; } else { LCD_DB4 = 0; } nibble >>= 1; 
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB5 = 1; } else { LCD_DB5 = 0; } nibble >>= 1; 
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB6 = 1; } else { LCD_DB6 = 0; } nibble >>= 1; 
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB7 = 1; } else { LCD_DB7 = 0; }  

  LCD_E = 1;
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
  LCD_E = 0;

  while( nsec > 0 ) {
    delay_us(1); 
    nsec--;
  }
} 

//-------------------------------------------------------------------------------
// tfBXvCɂPoCgM
//-------------------------------------------------------------------------------
void lcd_byte(unsigned char rs,unsigned char ch )
{
  unsigned char ch_bit;
  unsigned char ch_tmp;
  unsigned char i;
  
  ch_tmp = ch;
  
  if ( rs == 1 ) {
    LCD_RS = 1;
  } else {
    LCD_RS = 0;
  }

  for( i = 0; i <= 1 ; i++ ) {
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB7 = 0; } else { LCD_DB7 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB6 = 0; } else { LCD_DB6 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB5 = 0; } else { LCD_DB5 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB4 = 0; } else { LCD_DB4 = 1; } ch_tmp <<= 1;
    LCD_E = 1;
      Delay1TCY();
      Delay1TCY();
      Delay1TCY();
    LCD_E = 0;
  }  
  
  if (ch <= 0b00000011) {
    delay_us(1640);
  } else {
    delay_us(  46);
  }

  LCD_RS = 0;
}

//-------------------------------------------------------------------------------
// tfBXvCN
//-------------------------------------------------------------------------------
void lcd_clear()
{
  lcd_send_nibble( 0b0000,     2 );
  lcd_send_nibble( 0b0001,  1640 );
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\z[|WV
//-------------------------------------------------------------------------------
void lcd_home()
{
  lcd_byte( 0, 0b00000010 );
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\
//-------------------------------------------------------------------------------
void lcd_cursor( enum type_LCD_cursor cursor,  enum type_LCD_cursor blink )
{
  unsigned char ch;

  ch = 0b1100;
  if ( cursor == CURSOR_ON ) {
    ch = ch | 0b0010;
  }
  if ( blink == BLINK_ON ) {
    ch = ch | 0b0001;
  }

  lcd_send_nibble( 0b0000,   2 );
  lcd_send_nibble( ch    ,  46 );
}

//-------------------------------------------------------------------------------
// tfBXvC
//-------------------------------------------------------------------------------
void lcd_init()
{
  delay_ms(50);
  LCD_E_TRIS   = PORT_OUTPUT;
  LCD_RS_TRIS  = PORT_OUTPUT;
  LCD_DB7_TRIS = PORT_OUTPUT;
  LCD_DB6_TRIS = PORT_OUTPUT;
  LCD_DB5_TRIS = PORT_OUTPUT;
  LCD_DB4_TRIS = PORT_OUTPUT;

  LCD_E   = 0;
  LCD_RS  = 0;

  delay_ms(30);

  lcd_send_nibble( 0b0011, 4100 );
  lcd_send_nibble( 0b0011,  100 );
  
  lcd_send_nibble( 0b0010,   40 );  // function set
  lcd_send_nibble( 0b0010,   40 );
  lcd_send_nibble( 0b1000,   40 );

  lcd_cursor( CURSOR_OFF , BLINK_OFF );
  lcd_clear();
}

//-------------------------------------------------------------------------------
// tfBXvCɂP\
//-------------------------------------------------------------------------------
void lcd_char( unsigned char ch )
{
  lcd_byte( 1, ch );
}

//-------------------------------------------------------------------------------
// tfBXvCɕ\
//-------------------------------------------------------------------------------
void lcd_string( char *buf )
{
  while( *buf != 0x0 ) {
    lcd_char( *buf );
    buf++;
  }
}

//-------------------------------------------------------------------------------
// tfBXvCɒ萔\
//-------------------------------------------------------------------------------
void lcd_const_string( const rom char *buf )
{
  while( *buf != 0x0 ) {
    lcd_char( *buf );
    buf++;
  }
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\ʒuύX
//
// x -> cusor x posision     range 0 to 7
// y -> cusor y posision     range 0 to 1
//
// ex
//   lcd_locate( 0, 0 )  <- home posision (upper left)
//-------------------------------------------------------------------------------
void lcd_locate( unsigned char x, unsigned char y )
{
  unsigned char addr;

  addr = 0b10000000;
  if ( y == 1 ) {
    addr += 0x40;
  }
  addr += x;

  lcd_byte( 0, addr );
}

//-------------------------------------------------------------------------------
// 16bit / 16bit Z@@
//    荞݃[`̊ZƓZ[`gpȂ悤ɂ邽߂̕
//    荞݃[`ŊZƂ͂gp
//-------------------------------------------------------------------------------
#pragma udata BANK1=0x100
unsigned char dividend_16,dividend_16H; 
unsigned char divisor_16,divisor_16H;
unsigned char divans_16,divans_16H;
unsigned char divwork_16,divwork_16H;
unsigned char div_counter_16; 
unsigned char bsr_save_16;  

unsigned char dividend_32,dividend_32H,dividend_32HH,dividend_32HHH;
unsigned char divisor_32,divisor_32H,divisor_32HH,divisor_32HHH; 
unsigned char divans_32,divans_32H,divans_32HH,divans_32HHH;
unsigned char divwork_32,divwork_32H,divwork_32HH,divwork_32HHH;
unsigned char div_counter_32;
unsigned char bsr_save_32;
#pragma udata

unsigned short warizan_16x16(unsigned short dividend, unsigned short divisor)
{
  union {
    unsigned short Wval;
    unsigned char  Bval[2];
  } tmp;

  tmp.Wval = dividend;
  dividend_16  = tmp.Bval[0]; 
  dividend_16H = tmp.Bval[1];
  tmp.Wval = divisor; 
  divisor_16   = tmp.Bval[0]; 
  divisor_16H  = tmp.Bval[1]; 

        _asm
        MOVFF  BSR,bsr_save_16
        MOVLB  1    // bank1
MY_SB_DVD16X16:
        CLRF   divwork_16H,     BANKED
        CLRF   divwork_16,      BANKED
MY_SB_DVD16X162:
        MOVLW  16
        MOVWF  div_counter_16,  BANKED
MY_SB_DVD16X16LP:
        RLCF   dividend_16H,0,  BANKED
        RLCF   divwork_16,1,    BANKED
        RLCF   divwork_16H,1,   BANKED
        MOVF   divisor_16,0,    BANKED
        SUBWF  divwork_16,1,    BANKED
        MOVF   divisor_16H,0,   BANKED
        BTFSS  STATUS,0,        ACCESS
        INCFSZ divisor_16H,0,   BANKED
        SUBWF  divwork_16H,1,   BANKED
        BC     MY_SB_DVD16X16LP2
        MOVF   divisor_16,0,    BANKED
        ADDWF  divwork_16,1,    BANKED
        MOVF   divisor_16H,0,   BANKED
        BTFSC  STATUS,0,        ACCESS
        INCFSZ divisor_16H,0,   BANKED
        ADDWF  divwork_16H,1,   BANKED
        BCF    STATUS,0,        ACCESS
MY_SB_DVD16X16LP2:
        RLCF   dividend_16,1,   BANKED
        RLCF   dividend_16H,1,  BANKED
        DECFSZ div_counter_16,1,BANKED
        BRA    MY_SB_DVD16X16LP
        MOVF   dividend_16,0,   BANKED
        movwf  divans_16,       BANKED
        movff  dividend_16H,divans_16H
        MOVFF  bsr_save_16,BSR
        _endasm

  tmp.Bval[0] = divans_16;
  tmp.Bval[1] = divans_16H;

  return tmp.Wval;
}

//-------------------------------------------------------------------------------
// 32bit / 32bit Z@@
//    荞݃[`̊ZƓZ[`gpȂ悤ɂ邽߂̕
//    荞݃[`ŊZƂ͂gp
//-------------------------------------------------------------------------------
unsigned long warizan_32x32( unsigned long dividend, unsigned long  divisor)
{
  union {
    unsigned long LWval;
    unsigned char Bval[4];
  } tmp;

  tmp.LWval = dividend;
  dividend_32    = tmp.Bval[0]; 
  dividend_32H   = tmp.Bval[1]; 
  dividend_32HH  = tmp.Bval[2];
  dividend_32HHH = tmp.Bval[3]; 
  tmp.LWval = divisor; 
  divisor_32     = tmp.Bval[0]; 
  divisor_32H    = tmp.Bval[1]; 
  divisor_32HH   = tmp.Bval[2]; 
  divisor_32HHH  = tmp.Bval[3]; 
  
        _asm
        MOVFF  BSR,bsr_save_32
        MOVLB  1    // bank1
MY_SB_DIV_U_32X32:
        CLRF   divans_32,       BANKED
        CLRF   divans_32H,      BANKED
        CLRF   divans_32HH,     BANKED
        CLRF   divans_32HHH,    BANKED
        CLRF   divwork_32,      BANKED
        CLRF   divwork_32H,     BANKED
        CLRF   divwork_32HH,    BANKED
        CLRF   divwork_32HHH,   BANKED
        MOVF   divisor_32HHH,0, BANKED
        IORWF  divisor_32HH,0,  BANKED
        IORWF  divisor_32H,0,   BANKED
        IORWF  divisor_32,0,    BANKED
        BZ     MY_DIV_U_32X32_EXIT
        MOVLW  32
        MOVWF  div_counter_32,  BANKED
MY_DIV_U_32X32_LOOP:
        BCF    STATUS,0,      ACCESS
        RLCF   dividend_32,1,   BANKED
        RLCF   dividend_32H,1,  BANKED
        RLCF   dividend_32HH,1, BANKED
        RLCF   dividend_32HHH,1,BANKED
        RLCF   divwork_32,1,    BANKED
        RLCF   divwork_32H,1,   BANKED
        RLCF   divwork_32HH,1,  BANKED
        RLCF   divwork_32HHH,1, BANKED
        MOVF   divisor_32HHH,0, BANKED
        SUBWF  divwork_32HHH,0, BANKED
        BNZ    MY_L // $ + 18
        MOVF   divisor_32HH,0,  BANKED
        SUBWF  divwork_32HH,0,  BANKED
        BNZ    MY_L // $ + 12
        MOVF   divisor_32H,0,   BANKED
        SUBWF  divwork_32H,0,   BANKED
        BNZ    MY_L // $ + 6
        MOVF   divisor_32,0,    BANKED
        SUBWF  divwork_32,0,    BANKED
MY_L:
        BNC    MY_DIV_U_32X32_SKIP
        MOVF   divisor_32,0,    BANKED
        SUBWF  divwork_32,1,    BANKED
        MOVF   divisor_32H,0,   BANKED
        BTFSS  STATUS,0,      ACCESS
        INCFSZ divisor_32H,0,   BANKED
        SUBWF  divwork_32H,1,   BANKED
        MOVF   divisor_32HH,0,  BANKED
        BTFSS  STATUS,0,      ACCESS
        INCFSZ divisor_32HH,0,  BANKED
        SUBWF  divwork_32HH,1,  BANKED
        MOVF   divisor_32HHH,0, BANKED
        BTFSS  STATUS,0,      ACCESS
        INCFSZ divisor_32HHH,0, BANKED
        SUBWF  divwork_32HHH,1, BANKED
        BSF    STATUS,0,      ACCESS
MY_DIV_U_32X32_SKIP:
        RLCF   divans_32,1,     BANKED
        RLCF   divans_32H,1,    BANKED
        RLCF   divans_32HH,1,   BANKED
        RLCF   divans_32HHH,1,  BANKED
        DECFSZ div_counter_32,1,BANKED
        BRA    MY_DIV_U_32X32_LOOP
MY_DIV_U_32X32_EXIT:
        MOVFF  bsr_save_32,BSR
        _endasm   

  tmp.Bval[0] = divans_32;
  tmp.Bval[1] = divans_32H;
  tmp.Bval[2] = divans_32HH;
  tmp.Bval[3] = divans_32HHH;
  
  return tmp.LWval;
}

//-------------------------------------------------------------------------------
// ovlf[eB[̐ݒ@荞݃[`p   10bit      0 to 1023
//-------------------------------------------------------------------------------
void pwm_set_duty_inturrupt( unsigned short duty )
{
  if( Mmode.control == MMODE_BRAKE && Mmode.direction == CW ) {
    duty = 1023 - duty;
  }

  // duty LSB
  if ((duty & 0x0001) == 0x0001) {
    CCP1CONbits.DC1B0 = 1;
  } else {
    CCP1CONbits.DC1B0 = 0;
  }
  
  duty = duty >> 1;
  // duty LSB
  if ((duty & 0x0001) == 0x0001) {
    CCP1CONbits.DC1B1 = 1;
  } else {
    CCP1CONbits.DC1B1 = 0;
  }
  duty >>= 1;

  CCPR1L = duty;
}


//-------------------------------------------------------------------------------
// ovlf[eB[̐ݒ                 10bit      0 to 1023
//    WbN͊荞ݗpƑSB
//-------------------------------------------------------------------------------
void pwm_set_duty( unsigned short duty )
{
  if( Mmode.control == MMODE_BRAKE && Mmode.direction == CW ) {
    duty = 1023 - duty;
  }

  // duty LSB
  if ((duty & 0x0001) == 0x0001) {
    CCP1CONbits.DC1B0 = 1;
  } else {
    CCP1CONbits.DC1B0 = 0;
  }
  
  duty = duty >> 1;
  // duty LSB
  if ((duty & 0x0001) == 0x0001) {
    CCP1CONbits.DC1B1 = 1;
  } else {
    CCP1CONbits.DC1B1 = 0;
  }
  duty >>= 1;

  CCPR1L = duty;
}


//-------------------------------------------------------------------------------
// dbboiovlj
//-------------------------------------------------------------------------------
void motor_control_mode( enum type_control_mode mode )
{
  Mmode.control = mode;

  MOTOR_SELECT = 0;
  switch ( mode ) {
    case MMODE_FREE:
      CCP1CONbits.P1M0 = 1;    // Full-bridge
      MOTOR_SELECT = 1;
      PR2 = 0xFF;
      Timer2_prescale = 0b10;  // PWM Frequency 2.93kHz  Prescaler 1:16
      break;
    case MMODE_BRAKE:
      CCP1CONbits.P1M1 = 1;    // Half-bridge
      CCP1CONbits.P1M0 = 0;    // Half-bridge
      MOTOR_P1C = 0;           // inactive 
      MOTOR_P1D = 1;           // inactive
      MOTOR_SELECT = 1;
      PR2 = 0xFF;
      Timer2_prescale = 0b10;  // PWM Frequency 2.93kHz  Prescaler 1:16
      break;
    case MMODE_DUTY:
      CCP1CONbits.P1M1 = 1;    // Half-bridge
      CCP1CONbits.P1M0 = 0;    // Half-bridge
      MOTOR_SELECT = 0;
      PR2 = 0xFF;
      Timer2_prescale = 0b01;  // PWM Frequency 11.72kHz  Prescaler 1:4
      Mmode.direction = CW;
      break;
  }
  
  Timer2_enable = true;
  ECCP1DELbits.PDC  = 0b011111;     // pwm dead-band delay  6bit  number of Fsoc/4 cycles
}

//-------------------------------------------------------------------------------
// dbboiovlj̐ݒ
//-------------------------------------------------------------------------------
void motor_control_direction( enum type_motor_direction md )
{
  switch( Mmode.control ) {
    case  MMODE_BRAKE:
      switch( md ) {
        case CCW:
          MOTOR_P1C = 0;   // High-side inactive 
          MOTOR_P1D = 0;   // Low-side  active
          break;
        case CW:
          MOTOR_P1D = 1;   // Low-side  inactive 
          MOTOR_P1C = 1;   // High-side active
          break;
      }
      break;
    case MMODE_FREE:
      switch( md ) {
        case CCW: CCP1CONbits.P1M1 = 0; break;
        case CW:  CCP1CONbits.P1M1 = 1; break;
      }
      break;
  }
}

//-------------------------------------------------------------------------------
//  Motor RUN = PWM active
//-------------------------------------------------------------------------------
void motor_control_run(void)
{
  CCP1CONbits.CCP1M = 0b1101;    // PWM mode     P1A,P1C active High    P1B,P1D active Low
}

//-------------------------------------------------------------------------------
//  Motor STOP = PWM inactive
//-------------------------------------------------------------------------------
enum type_brake{ FREE, BRAKE };

void motor_control_stop(enum type_brake brake)
{
  CCP1CONbits.CCP1M = 0b0000;   // PWM off    
  
  switch( brake ) {
    case FREE:
      MOTOR_SELECT = 1;   // select B channel
      MOTOR_P1A = 0;
      MOTOR_P1B = 1;
      MOTOR_P1C = 0;   // via 74HC157 (buffer)
      MOTOR_P1D = 1;   // via 74HC157 (buffer)
      break;
    case BRAKE:
      MOTOR_SELECT = 1;  // select B channel
      MOTOR_P1A = 1;
      MOTOR_P1B = 1;
      MOTOR_P1C = 1;  // via 74HC157 (buffer)
      MOTOR_P1D = 1;  // via 74HC157 (buffer)
      break;
  }
}

//-------------------------------------------------------------------------------
// ݉]ƖڕW]Ƃ̌덷\
//-------------------------------------------------------------------------------
void display_rpm_div(void)
{
  char buf[6];
  unsigned long  rpm_long;
  unsigned short rpm_short;
  unsigned short rpm_div;
    
  rpm_long = TMR_COUNT_60SEC / Raw_tmr_kaitenkei.Lval;
  rpm_short = (unsigned short)rpm_long;
    
  if ( rpm_short < Target_rpm ) {
    rpm_div = Target_rpm - rpm_short;
  } else {
    rpm_div = rpm_short - Target_rpm;
  }
  sprintf( buf, (const far rom char *)"%5d", rpm_div );
  lcd_locate(0,0);
  lcd_string(buf);
  lcd_const_string("rpm");
}

//-------------------------------------------------------------------------------
// ݉]̕\
//-------------------------------------------------------------------------------
void display_current_rpm(void)
{
  char buf[6];
  unsigned long  rpm_long;
  unsigned short rpm_short;
  unsigned short rpm_div;
 
  rpm_long = TMR_COUNT_60SEC / Raw_tmr_kaitenkei.Lval;
  rpm_short = (unsigned short)rpm_long;
  
  sprintf( buf, (const far rom char *)"%5d", rpm_short );
  lcd_locate(0,0);
  lcd_string(buf);
  lcd_const_string("rpm");
 
 /* 
  if( rpm_short < Target_rpm_minus ) {
    LED2 = LED_ON;
  } else {
    LED2 = LED_OFF;
  }
  
  if( Target_rpm_plus < rpm_short ) {
    LED3 = LED_ON;
  } else {
    LED3 = LED_OFF;
  }
*/  
}

//-------------------------------------------------------------------------------
// ڕW]tfBXvCɕ\
//-------------------------------------------------------------------------------
void display_target_rpm(void)
{
  char buf[6];

  sprintf( buf, (const far rom char *)"%5d", Target_rpm );
  lcd_locate(0,0);
  lcd_string(buf);
}

//-------------------------------------------------------------------------------
// f[eB[̂kbc\  O`POQR
//-------------------------------------------------------------------------------
void display_pwm_duty(void)
{
  char buf[6];

  sprintf( buf, (const far rom char *)"%5d", Pwm_duty );
  lcd_locate(0,1);
  lcd_string(buf);
}

//-------------------------------------------------------------------------------
// QoCgϐPUiłkbcɕ\ (oOp֐debug routine)
//-------------------------------------------------------------------------------
void display_short( unsigned short arg, unsigned char row )
{
  char buf[5];

  lcd_locate(0,row);  
  sprintf( buf, (const far rom char *)"%04x", arg );
  lcd_string(buf);
}

//****************************************************************************
// Dx 荞݃[`@    ]ׂ̐͂Ă̒ŎsĂ
//****************************************************************************
static unsigned long  rpm_prev;

void High_priority_interrupt ()
{
  long_union raw_tmr_control;  // kaitenkei
  unsigned long  rpm_now_LWord;
  unsigned short rpm_now_Word;
  unsigned long worklng;
  unsigned char dummy;
  unsigned short tmp_word,tmp_word2;

  if ( sensor_A_changed == true ) {
    Timer0_enable = timer_stop;
    if ( Timer0_overflowd == true ) {
      TMR0_over_flow_count++;
    }
    raw_tmr_control.Bval[0] = TMR0L;
    raw_tmr_control.Bval[1] = TMR0H;
    raw_tmr_control.Wval[1] = TMR0_over_flow_count;
    TMR0H = 0;
    TMR0L = 0;
    Timer0_overflowd = false;
    Timer0_enable = timer_start;
    TMR0_over_flow_count = 0;

    if ( kaitenkei_stat == GET_TIMER ) {
      Raw_tmr_kaitenkei.Lval = raw_tmr_control.Lval; 
      kaitenkei_stat = DISPLAY_RPM;
    }

    worklng = warizan_32x32(TMR_COUNT_60SEC,raw_tmr_control.Lval);
    rpm_now_LWord = worklng;

    rpm_now_Word = rpm_now_LWord;

if( rpm_now_Word < Target_rpm_minus ) {
  LED2 = LED_ON;
} else {
  LED2 = LED_OFF;
}
  
if( Target_rpm_plus < rpm_now_Word ) {
  LED3 = LED_ON;
} else {
  LED3 = LED_OFF;
}

    if( Mmode.start == START ) {
	    //-----  --------------------------
      if( Target_rpm < rpm_now_Word ) {
        if( rpm_now_Word > rpm_prev ) {
          tmp_word2 = warizan_16x16( rpm_now_Word - Target_rpm, Rpm_per_duty);
          if( tmp_word2 > 7  ) {
	          tmp_word2 = 7;
	        }  
          if( tmp_word < Pwm_duty ) {
            Pwm_duty -= tmp_word;
          }
        }
      } else {
	    //-----  --------------------------
        if( rpm_now_Word < rpm_prev ) {
          tmp_word2 = warizan_16x16( (Target_rpm - rpm_now_Word), Rpm_per_duty);
          if( tmp_word2 > 7  ) {
	          tmp_word2 = 7;
	        }
          tmp_word = Pwm_duty + tmp_word2;
          if( tmp_word < Max_duty_limit  ) {
            Pwm_duty = tmp_word;
          } else {
            Pwm_duty = Max_duty_limit;
          }
        }
      }
      rpm_prev = rpm_now_Word;
      pwm_set_duty_inturrupt(Pwm_duty);
    }

    dummy = PORTB;
    sensor_A_changed = false;
    Motor_stall_counter = 0;
  }  //-------- INT0 interrupt ------- E N D ------------

  //---- TMR0 overflow ----------
  if ( Timer0_overflowd == true ) {
    Timer0_overflowd = false;
    TMR0_over_flow_count++;
    if( Mmode.start == START ) {
      if ( TMR0_over_flow_count > Motor_stall_th ) {
        if ( Pwm_duty < MAX_DUTY ) {
          Pwm_duty += 30;
        }
        TMR0_over_flow_count = 0;
        pwm_set_duty_inturrupt(Pwm_duty);
      }
    }
  }
}

//****************************************************************************
// Dx@荞݃[`
//****************************************************************************
void Low_priority_interrupt ()
{
  //---- TMR3 overflow (1~bɂslqR͂ӂ) ----------
  if ( Timer3_overflowd == true ) {
    Timer3_overflowd = false;
    TMR3H = 0xD1;
    TMR3L = 0x20;

    if ( kaitenkei_stat == DISPLAY_WAIT ) {
      if ( Display_interval_counter < DISPLAY_INTERVAL ) {
        Display_interval_counter++;
      } else {
        Display_interval_counter = 0;
        kaitenkei_stat = GET_TIMER;
        Display_timeout_counter = DISPLAY_TIMEOUT;
      }
    }

    //--------- `^Oh~ ---------------------------------
    if( Sw_timer > 0 ) {
      Sw_timer--;
    }
    
    //--------- ݉]̕\Ԋup -------------------------
    if( Display_change_rpm_counter > 0 ) {
      Display_change_rpm_counter--;
    }

    //--------- ݉]̃[\ ---------------------------
    if( Display_timeout_counter > 0 ) {
      Display_timeout_counter--;
    }
    
    //--------- ovlf[eB[̕\Ԋup@---------------------
    if( Display_duty_counter > 0 ) {
      Display_duty_counter--;
    }
    
    //--------- [^[ً}~ -------------------------------
    if( Mmode.start == START ) {
      if( Motor_stall_counter < MAX_STALL_TIME ) {
        Motor_stall_counter++;
      }
      if( Motor_stall_counter == MAX_STALL_TIME ) {
        Mmode.start = STOP;
        motor_control_stop(FREE); 
        LED1 = LED_OFF; 
        LED3 = LED_ON;
      }
    }  
  }
}


//----------------------------------------------------------------------------
// [^[GR[_Tu[`
//----------------------------------------------------------------------------
struct {
  unsigned a_port:1;
  unsigned b_port:1;
  unsigned a_lock:1;
  unsigned b_lock:1;
  unsigned a_tmp:1;
  unsigned b_tmp:1;
} Re_stat;

void rotary_encoder_init()
{
  RENCODER_A_TRIS = PORT_INPUT;
  RENCODER_B_TRIS = PORT_INPUT;

  Re_stat.a_port = RENCODER_A;
  Re_stat.b_port = RENCODER_B;

  Re_stat.a_lock = false;
  Re_stat.b_lock = false;
}

enum type_encoder_direction rotary_encoder()
{
  enum type_encoder_direction rotary_stat;  

  Re_stat.a_tmp = RENCODER_A;
  Re_stat.b_tmp = RENCODER_B;
 
  rotary_stat  = ENCODER_STOP;

  //--- CW ----------------
  if ((Re_stat.a_port == 1) && (Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 1)) {
    Re_stat.a_lock = true;
  }
  if (Re_stat.a_lock == true ) {
    if ((Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 0) ) {
      Re_stat.a_lock = false;
      rotary_stat = CW;
    }
  }

  //--- CCW ----------------
  if ((Re_stat.b_port == 1) && (Re_stat.b_tmp == 0) && (Re_stat.a_tmp == 1)) {
    Re_stat.b_lock = true;
  }
  if (Re_stat.b_lock == true ) {
    if ((Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 0) ) {
      Re_stat.b_lock = false;
      rotary_stat = CCW;
    }
  }
  
  Re_stat.a_port = Re_stat.a_tmp;
  Re_stat.b_port = Re_stat.b_tmp;

  return rotary_stat;       // CW / CCW / ENCODER_STOP
}

//----------------------------------------------------------------------------
// ȂQoCg@t\
//----------------------------------------------------------------------------
void display_ushort( unsigned short val, unsigned char length, unsigned char col, unsigned char row ) 
{
  char buf[9];
  lcd_locate(col,row);
  switch( length ) {
    case 1: sprintf( buf, (rom char far const *)"%1d", val ); break;
    case 2: sprintf( buf, (rom char far const *)"%2d", val ); break;
    case 3: sprintf( buf, (rom char far const *)"%3d", val ); break;
    case 4: sprintf( buf, (rom char far const *)"%4d", val ); break;
    case 5: sprintf( buf, (rom char far const *)"%5d", val ); break;
 }
 lcd_string( buf );
}   

//----------------------------------------------------------------------------
// ׂ
//----------------------------------------------------------------------------
unsigned short pow_short( unsigned short val, unsigned char pv )
{
  unsigned char i;
  unsigned short return_val;

  if( pv == 0 ) {
    return 1;
  }
  
  return_val = 1;
  for( i = 0 ; i < pv ; i++ ) {
    return_val *= val;
  } 

  return return_val;
}

//----------------------------------------------------------------------------
// [^[GR[_ɂTl
//
// init_val:l 
// min_val:͉\ȍŏl 
// max_val:͉\ȍől 
//----------------------------------------------------------------------------
unsigned short scan_5digit(
 unsigned short init_val,
 unsigned short min_val,
 unsigned short max_val )
{
  unsigned char turn;     // CW,CCW@[^[GR[_]
  unsigned char cursor;   // 0@to 4@J[\ʒu
  unsigned short digit;   // ͂li[ϐ
  char buf[6];            // t\pobt@
    
  cursor = 0;  
  digit = init_val;
  while(PUSH_SW1 == SW_OFF) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if( turn == CW ) {
        if( digit != max_val ) {
          beep_pi();
        } 
        if( digit + pow_short(10,cursor) > max_val ) {
          digit = MAX_RPM;
        } else {
          digit += pow_short(10,cursor);
        }
      } else {
        if( digit != min_val ) {
          beep_pi();
        }         
        if( digit < ( pow_short(10,cursor) + min_val ) ) {
          digit = min_val;
        } else {
          digit -= pow_short(10,cursor);
        }
      }
      
      sprintf( buf,(rom char far const *)"%5d", digit  );
      lcd_locate(0,1);
      lcd_string( buf );
      lcd_locate(4-cursor,1);
    }

    if( PUSH_SW2 == SW_ON && Sw2_status == SW_ENABLE) {
      beep_pi();
      if( cursor == 4 ) {
        cursor = 0;
      } else {
        cursor++;
      }
      lcd_locate(4-cursor,1);
      Sw2_status = SW_DISABLE;
      Sw_timer = 30;    // decliment interrupt routine
    }

    if( PUSH_SW2 == SW_OFF && Sw_timer == 0 ) {
      Sw2_status = SW_ENABLE;
    }
  }
  
  return digit;
}  


//----------------------------------------------------------------------------
//@]̐ݒ@@@]JnXCb`irvQjɂ̉]ɂȂ
//----------------------------------------------------------------------------
void set_initial_rpm(void)
{
  char buf[6];
  short_union rpm;

  rpm.Wval = 0;
  rpm.Bval[0] = Read_b_eep(eeaddr_initial_rpm_0);
  rpm.Bval[1] = Read_b_eep(eeaddr_initial_rpm_1);

  Sw2_status = SW_ENABLE;
  sprintf( buf,(rom char far const *)"%5d", rpm.Wval );
  lcd_clear();
  lcd_const_string("Initial ");
  lcd_locate(5,1);
  lcd_const_string("RPM");
  display_ushort( rpm.Wval, 5, 0, 1 );

  lcd_locate(4,1);
  lcd_cursor(CURSOR_ON,BLINK_OFF);
 
  rpm.Wval = scan_5digit( rpm.Wval, MIN_RPM, MAX_RPM ); 

  lcd_cursor(CURSOR_OFF,BLINK_OFF);
  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_initial_rpm_0, rpm.Bval[0] );
  Write_b_eep( eeaddr_initial_rpm_1, rpm.Bval[1] );
  delay_ms(1000);
}

//----------------------------------------------------------------------------
//@ݒ̍ő]@@@ۂɂ̓[^[̐\EœłɂȂ邱Ƃ
//----------------------------------------------------------------------------
void set_max_rpm(void)
{
  char buf[6];
  short_union rpm;

  rpm.Wval = 0;
  rpm.Bval[0] = Read_b_eep(eeaddr_max_rpm_0);
  rpm.Bval[1] = Read_b_eep(eeaddr_max_rpm_1);

  Sw2_status = SW_ENABLE;
  sprintf( buf,(rom char far const *)"%5d", rpm.Wval );
  lcd_clear();
  lcd_const_string("MAX");
  lcd_locate(5,1);
  lcd_const_string("RPM");
  display_ushort( rpm.Wval, 5, 0, 1 );

  lcd_locate(4,1);
  lcd_cursor(CURSOR_ON,BLINK_OFF);
 
  rpm.Wval = scan_5digit( rpm.Wval, MIN_RPM, MAX_RPM ); 

  lcd_cursor(CURSOR_OFF,BLINK_OFF);
  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_max_rpm_0, rpm.Bval[0] );
  Write_b_eep( eeaddr_max_rpm_1, rpm.Bval[1] );
  delay_ms(1000);
}

//----------------------------------------------------------------------------
//@ݒ̍ŏ]@@@ۂɂ̓[^[̐\EœłɂȂ邱Ƃ
//----------------------------------------------------------------------------
void set_min_rpm(void)
{
  char buf[6];
  short_union rpm;

  rpm.Wval = 0;
  rpm.Bval[0] = Read_b_eep(eeaddr_min_rpm_0);
  rpm.Bval[1] = Read_b_eep(eeaddr_min_rpm_1);

  Sw2_status = SW_ENABLE;
  sprintf( buf,(rom char far const *)"%5d", rpm.Wval );
  lcd_clear();
  lcd_const_string("MIN");
  lcd_locate(5,1);
  lcd_const_string("RPM");
  display_ushort( rpm.Wval, 5, 0, 1 );

  lcd_locate(4,1);
  lcd_cursor(CURSOR_ON,BLINK_OFF);
 
  rpm.Wval = scan_5digit( rpm.Wval, MIN_RPM, MAX_RPM ); 

  lcd_cursor(CURSOR_OFF,BLINK_OFF);
  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();
  Write_b_eep( eeaddr_min_rpm_0, rpm.Bval[0] );
  Write_b_eep( eeaddr_min_rpm_1, rpm.Bval[1] );
  delay_ms(1000);
}

//----------------------------------------------------------------------------
//@GR[_[񂵂Ƃ̑
//----------------------------------------------------------------------------
void set_step_rpm(void)
{
  char buf[6];
  short_union rpm;

  rpm.Wval = 0;
  rpm.Bval[0] = Read_b_eep(eeaddr_step_rpm_0);
  rpm.Bval[1] = Read_b_eep(eeaddr_step_rpm_1);

  Sw2_status = SW_ENABLE;
  sprintf( buf,(rom char far const *)"%5d", rpm.Wval );
  lcd_clear();
  lcd_const_string("STEP");
  lcd_locate(5,1);
  lcd_const_string("RPM");
  display_ushort( rpm.Wval, 5, 0, 1 );

  lcd_locate(4,1);
  lcd_cursor(CURSOR_ON,BLINK_OFF);
 
  rpm.Wval = scan_5digit( rpm.Wval, MIN_RPM, MAX_RPM ); 

  lcd_cursor(CURSOR_OFF,BLINK_OFF);
  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_step_rpm_0, rpm.Bval[0] );
  Write_b_eep( eeaddr_step_rpm_1, rpm.Bval[1] );
  delay_ms(1000);
}

//----------------------------------------------------------------------------
// dd^[^[id@с@őf[eB[̐ݒ
//----------------------------------------------------------------------------
void display_voltage( unsigned char voltage )
{
  lcd_locate(6,1);
  lcd_char((voltage % 10)+'0');
  voltage /= 10;
  lcd_locate(5,1);
  lcd_char('.');
  lcd_locate(4,1);
  lcd_char((voltage % 10)+'0');
  voltage /= 10;
  lcd_locate(3,1);
  if( voltage != 0 ) {
    lcd_char(voltage+'0');
  } else {
    lcd_char(voltage+' ');
  }
}

void set_voltage(void)
{
  unsigned char supply_voltage;
  unsigned char motor_voltage;
  unsigned char turn;     // CW,CCW
  char          max_duty_str[5];
  long_union    max_duty;
  unsigned long tmp;

  lcd_clear();
  lcd_locate(7,1);
  lcd_char('V');

  //------------ dd̐ݒ -------------------------
  lcd_locate(0,0);
  lcd_const_string("Dengen  ");

  supply_voltage = Read_b_eep(eeaddr_supply_voltage);
  while( PUSH_SW1 == SW_OFF ) {
    display_voltage(supply_voltage);
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if( turn == CW ) {
        if( supply_voltage < MAX_SUPPLY_VOLTAGE ) {
          supply_voltage++;
          beep_pi();
        }
      } else {  // CCW
        if( supply_voltage > MIN_SUPPLY_VOLTAGE ) {
          supply_voltage--;
          beep_pi();
        }
      }
    }
  }

  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_supply_voltage, supply_voltage );
  delay_ms(1000);

  //------------ [^[d̐ݒ -------------------------
  lcd_locate(0,0);
  lcd_const_string("Motor   ");

  motor_voltage = Read_b_eep(eeaddr_motor_voltage);
  while( PUSH_SW1 == SW_OFF ) {
    display_voltage(motor_voltage);
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if( turn == CW ) {
        if( motor_voltage < supply_voltage ) {
          motor_voltage++;
          beep_pi();
        }
      } else {  // CCW
        if( motor_voltage > MIN_MOTOR_VOLTAGE ) {
          motor_voltage--;
          beep_pi();
        }
      }
    }
  }

  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();
  Write_b_eep( eeaddr_motor_voltage, motor_voltage );
  delay_ms(1000);

  //------------ őf[eB[̐ݒ -------------------------
  // [^[idƓdd玩vZĂ
  lcd_clear();
  lcd_const_string("Max Duty");
  max_duty.Lval = 1023 * (unsigned long)motor_voltage;
  max_duty.Lval /= (unsigned long)supply_voltage;
  tmp = (unsigned long)max_duty.Wval[0];
  tmp = tmp * 100 / 1023; 
  itoa( tmp, max_duty_str );
  lcd_locate(0,1);
  lcd_string( max_duty_str );
  lcd_char('%');
  delay_ms(750);
  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();
  Write_b_eep( eeaddr_max_duty_0, max_duty.Bval[0] );
  Write_b_eep( eeaddr_max_duty_1, max_duty.Bval[1] );
  delay_ms(1000); 
}

//----------------------------------------------------------------------------
// [^[䃂[h̐ݒ
//----------------------------------------------------------------------------
void display_mode(enum type_control_mode mode)
{
  lcd_locate(0,1);
  switch( mode ) {
    case MMODE_FREE:  lcd_const_string("FREE    "); break;
    case MMODE_BRAKE: lcd_const_string("BRAKE   "); break;
    case MMODE_DUTY:  lcd_const_string("DUTY    "); break;
  }
}

void set_mode(void)
{
  enum type_control_mode motor_control_mode;
  unsigned char turn;     // CW,CCW

  motor_control_mode = Read_b_eep(eeaddr_control_mode);
  lcd_clear();
  lcd_const_string("Mode    ");

  while( PUSH_SW1 == SW_OFF ) {
    display_mode( motor_control_mode );
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        switch( motor_control_mode ) {
          case MMODE_FREE:  motor_control_mode = MMODE_BRAKE; beep_pi(); break;
//        case MMODE_BRAKE: motor_control_mode = MMODE_DUTY;  beep_pi(); break;
        }
      } else {  // CCW
        switch( motor_control_mode ) {
//        case MMODE_DUTY:  motor_control_mode = MMODE_BRAKE; beep_pi(); break;
          case MMODE_BRAKE: motor_control_mode = MMODE_FREE;  beep_pi(); break;
        }
      }
    }
  }

  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_control_mode, motor_control_mode );
  delay_ms(1000);
}


//----------------------------------------------------------------------------
// [^[]̐ݒ
//----------------------------------------------------------------------------
void display_direction( enum type_motor_direction direction )
{
  lcd_locate(0,1);
  switch( direction ) {
    case CW:  lcd_const_string("CW "); break;
    case CCW: lcd_const_string("CCW"); break;
  }
}

void set_motor_direction(void)
{
  enum type_motor_direction direction;
  unsigned char turn;     // CW,CCW

  lcd_clear();
  lcd_const_string("KAITEN  ");
  direction = Read_b_eep(eeaddr_motor_direction);
  while( PUSH_SW1 == SW_OFF ) {
    display_direction( direction );
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        if( direction == CW ) {
          direction = CCW;
          beep_pi();
        }
      } else {  // CCW
        if( direction == CCW ) {
          direction = CW;
          beep_pi();
        }
      }
    }
  }    

  lcd_locate(0,0);
  lcd_const_string("--SAVE--");
  beep_pi();  
  Write_b_eep( eeaddr_motor_direction, direction );
  delay_ms(1000);
}


//----------------------------------------------------------------------------
// C[`
//----------------------------------------------------------------------------
void main ()
{
  short_union                 tmp_short;                
  enum type_encoder_direction turn;

  unsigned short              rpm_step;
  unsigned short              rpm_min;
  unsigned short              rpm_max;

  ADCON1bits.PCFG = 0b1111;  // digital I/O
  CMCONbits.CM    = 0b111;   // compatator off
  INTCON2bits.NOT_RBPU = 1;  // PortB pull-up disable

  sensor_A_int_enable        = true;  
  sensor_A_int_rising_edge   = true;

  sensor_B_int_enable        = false;  //@̃vOł͖gp
  sensor_B_int_rising_edge   = true;
  sensor_B_int_priority_high = true;

  SENSOR_A_TRIS = PORT_INPUT;
  SENSOR_B_TRIS = PORT_INPUT;

  PUSH_SW1_TRIS = PORT_INPUT;
  PUSH_SW2_TRIS = PORT_INPUT;
  SLIDE_SW_TRIS = PORT_INPUT;

  BUZZER_TRIS = PORT_OUTPUT;
  buzzer_off();

  LED1_TRIS = PORT_OUTPUT;
  LED2_TRIS = PORT_OUTPUT;
  LED3_TRIS = PORT_OUTPUT;

  Mmode.start    = STOP;
//  Mmode.run_mode = KEISOKU;
  Mmode.run_mode = RUN;
  motor_control_stop(FREE);  

  MOTOR_P1A_TRIS    = PORT_OUTPUT;
  MOTOR_P1B_TRIS    = PORT_OUTPUT;
  MOTOR_P1C_TRIS    = PORT_OUTPUT;
  MOTOR_P1D_TRIS    = PORT_OUTPUT;
  MOTOR_SELECT_TRIS = PORT_OUTPUT;
  TTL74HC157_G_TRIS = PORT_OUTPUT;
  TTL74HC157_G = TTL74HC157_ENABLE;

  LED1 = LED_OFF;
  LED2 = LED_OFF;
  LED3 = LED_OFF;

  rotary_encoder_init();
  lcd_init();

  //---- TMR0(16bitMODE) initialize --------------
  // Timer0_prescale_bit2 = 0   // PreScaler 1:2
  // Timer0_prescale_bit1 = 0   // PreScaler 1:2
  // Timer0_prescale_bit0 = 0   // PreScaler 1:2
  Timer0_prescale_NOT_assign = true;  
  Timer0_mode_8bit           = false;  // 16bit mode
  Timer0_interrupt_enable    = true;
  Timer0_use_external_clock  = false;
  Timer0_enable              = true;
  Timer0_priority            = High_priority;

  //---- TMR1(16bit) initialize (slqP͖gp) -----------------
  // Timer1_peripheral_int_enable = true;
  Timer1_interrupt_enable    = false; 
  Timer1_prescale_bit0       = 0;
  Timer1_prescale_bit1       = 0; 
  Timer1_use_external_clock  = false;
  Timer1_enable              = false;
  Timer1_priority            = Low_priority;

  //---- TMR2(16bit) initialize -----------------
  Timer2_interrupt_enable    = false; 

  //---- TMR3(16bitMODE) initialize -----------------
  Timer3_interrupt_enable    = true; 
  Timer3_prescale_bit0       = 0;
  Timer3_prescale_bit1       = 0;
  Timer3_mode_16bit          = true;
  Timer3_use_external_clock  = false;
  Timer3_not_sync_ext_clock  = true;
  Timer3_priority            = Low_priority;
  Timer3_enable              = true;
  TMR3H = 0xD1;   // 1ms
  TMR3L = 0x20;   // 1ms

  LED1 = LED_ON;
  LED2 = LED_ON;
  LED3 = LED_ON;
  buzzer_on();
  delay_ms(100);
  buzzer_off();
  LED1 = LED_OFF;
  LED2 = LED_OFF;
  LED3 = LED_OFF;

  lcd_clear();
  lcd_const_string("PROGRAM");
  lcd_locate(0,1);
  lcd_const_string( VERSION );
  delay_ms(500);
  
// test_run();

  //---- XChXCb`ɐݒ肷ƃZbgAbv[hɂȂ ---------------------
  if( SLIDE_SW == SW_ON ) {
    set_voltage();                    // Supply 5.0V to 12.0V   Motor 1.0 to Supply
    while( PUSH_SW1 == SW_ON ) { ; }
    set_max_rpm();                    //
    while( PUSH_SW1 == SW_ON ) { ; }
    set_min_rpm();                    //
    while( PUSH_SW1 == SW_ON ) { ; }
    set_initial_rpm();                // 
    while( PUSH_SW1 == SW_ON ) { ; }
    set_step_rpm();                   // 
    while( PUSH_SW1 == SW_ON ) { ; }
    set_mode();                       // Free,Brake,Duty
    while( PUSH_SW1 == SW_ON ) { ; }
    set_motor_direction();
    lcd_clear();
    lcd_const_string("SETUP   ");
    lcd_locate(0,1);
    lcd_const_string("COMPLETE");
    stop();
    // XCb`Oɂēd꒼ȂƐݒ胂[h͔Ȃ
  }

  //----------- ddoqnlݒlǂݏo -------------------------
  // ڕW]]
  tmp_short.Bval[0] = Read_b_eep(eeaddr_initial_rpm_0);
  tmp_short.Bval[1] = Read_b_eep(eeaddr_initial_rpm_1);
  Target_rpm = tmp_short.Wval;
  Target_rpm_minus = Target_rpm - RPM_DIV;
  Target_rpm_plus  = Target_rpm + RPM_DIV;
  // ŏ]
  tmp_short.Bval[0] = Read_b_eep(eeaddr_min_rpm_0);
  tmp_short.Bval[1] = Read_b_eep(eeaddr_min_rpm_1);
  rpm_min = tmp_short.Wval;
  Motor_stall_th = 13183 / rpm_min;
  // ő]
  tmp_short.Bval[0] = Read_b_eep(eeaddr_max_rpm_0);
  tmp_short.Bval[1] = Read_b_eep(eeaddr_max_rpm_1);
  rpm_max = tmp_short.Wval;
  // ]
  tmp_short.Bval[0] = Read_b_eep(eeaddr_step_rpm_0);
  tmp_short.Bval[1] = Read_b_eep(eeaddr_step_rpm_1);
  rpm_step = tmp_short.Wval;    
  //@f[eB[l
  tmp_short.Bval[0] = Read_b_eep( eeaddr_max_duty_0 );
  tmp_short.Bval[1] = Read_b_eep( eeaddr_max_duty_1 );
  Max_duty_limit = tmp_short.Wval;
  // [^[䃂[h Free/Brake/Duty
  Mmode.control     = Read_b_eep( eeaddr_control_mode );
  // ]
  Mmode.direction   = Read_b_eep( eeaddr_motor_direction );
delay_ms(100);
lcd_clear();

  //---- 荞݋ ---------------------------------------------
  RCONbits.IPEN   = 1;          // 1:enable 0:disable priority levels
  INTCONbits.GIEH = 1;          // 1:enable 0:disable interrupts
  INTCONbits.GIEL = 1;          // 1:enable 0:disable interrupts

  //--------- [^[䃂[h\ -----------------------------------
  lcd_clear();
  lcd_const_string("    0rpm");
  lcd_locate(7,1);
  switch( Mmode.control ) {
    case MMODE_FREE:  lcd_char('F'); break;   // Free
    case MMODE_BRAKE: lcd_char('B'); break;   // Brake
    case MMODE_DUTY:  lcd_char('D'); break;   // Duty
  }

  Rpm_per_duty = 13;  // RS-380
//  Rpm_per_duty = 70;  // FA-130
//  Rpm_per_duty = 32;  // FA-130
  Pwm_duty = Target_rpm / Rpm_per_duty;

  Display_interval_counter   = 0;
  Display_change_rpm_counter = 0;
  Display_timeout_counter    = 0;
  Display_duty_counter       = 0;
  Motor_stall_counter        = 0;
  kaitenkei_stat = DISPLAY_WAIT;
  while(1) {
    //----- ]Jn^~ ------------------------------------------
    if( PUSH_SW2 == SW_ON && Sw2_status == SW_ENABLE ) {
      if( Mmode.start == STOP ) {
        Mmode.start = START;
        Motor_stall_counter = 0;
        motor_control_mode( Mmode.control );
        motor_control_direction( Mmode.direction );
        motor_control_run();
        LED1 = LED_ON;
        LED3 = LED_OFF;
      } else {
        Mmode.start = STOP;
        motor_control_stop(FREE); 
        LED1 = LED_OFF;
      }
      Sw2_status = SW_DISABLE;
      Sw_timer = 100;    // msec   decliment interrupt routine
    }

    if( PUSH_SW2 == SW_OFF && Sw_timer == 0 ) {
      Sw2_status = SW_ENABLE;
    }    
    
    //----- ݉]̕\ ------------------------------------------
    if (kaitenkei_stat == DISPLAY_RPM ) {
      if( Display_change_rpm_counter == 0 ) { 
        display_current_rpm();
      }
      kaitenkei_stat = DISPLAY_WAIT;
    } else {
      if( Display_timeout_counter == 0 && Display_change_rpm_counter == 0 ) {
        lcd_locate(0,0);
        lcd_const_string("    0");
      }
    }
    
    //----- ovlf[eB[Ɖ]̕\bbv̎ɂ͂kdcQij_ ------
    if( Display_duty_counter == 0 ) {
      display_pwm_duty();
      Display_duty_counter = DISPLAY_DUTY_INTERVAL;
    }

    //----- GR[_[񂵂ĕϑ@----------------------------------------
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) { 
      if ( turn == CW ) {
        if( Target_rpm != rpm_max ) {
          beep_pi(); 
        }
        if( Target_rpm + rpm_step > rpm_max ) { 
          Target_rpm = rpm_max;
        } else {
          Target_rpm += rpm_step;
        }
      } else {   // CCW
        if( Target_rpm != rpm_min ) {
          beep_pi(); 
        }
        if( Target_rpm > (rpm_step + rpm_min) ) {
          Target_rpm -= rpm_step;
        } else {
          Target_rpm = rpm_min;
        }
      }
  Target_rpm_minus = Target_rpm - RPM_DIV;
  Target_rpm_plus  = Target_rpm + RPM_DIV;
      display_target_rpm();
      Display_change_rpm_counter = DISPLAY_CHANGE_RPM_INTERVAL;
    }
  }
}

//================================================================= E N D =========
